"use client";
import Link from "next/link";
import { useState } from "react";
import { useFormState } from "react-dom";

import { UserData } from "@snip/database/types";

import { signupSubmitAction } from "./action";

import { Form, FormHeader } from "components/form";
import { FormError } from "components/form/alert";
import { SecondaryButton, SubmitButton } from "components/form/buttons";
import { EmailInput, PasswordInput } from "components/form/inputs/text";
import { validateEmail, validatePW } from "components/form/validation";

export default function SignUp() {
    return (
        <div className="d-flex flex-grow-1 justify-content-center align-items-center">
            <SignupForm />
        </div>
    );
}

const initialState: {
    message?: string;
    userData?: UserData;
} = {};

type valid = "unset" | "valid" | "invalid";

export interface Validation {
    email: valid;
    password: valid;
    confirmPassword: valid;
}

function SignupForm() {
    const [state, formAction] = useFormState(signupSubmitAction, initialState);

    const [valid, setValid] = useState<Validation>({
        email: "unset",
        password: "unset",
        confirmPassword: "unset",
    });

    const validateForm = (formData: FormData) => {
        const valid = {
            email: "unset",
            password: "unset",
            confirmPassword: "unset",
        } as Validation;

        if (formData.has("email")) {
            valid["email"] = validateEmail(formData.get("email")! as string)
                ? "valid"
                : "invalid";
        }
        if (formData.has("password")) {
            valid["password"] = validatePW(formData.get("password")! as string)
                ? "valid"
                : "invalid";
            if (formData.has("confirmPassword")) {
                valid["confirmPassword"] =
                    formData.get("password") === formData.get("confirmPassword")
                        ? "valid"
                        : "invalid";
            }
        }

        // Update state
        setValid(valid);

        // If all fields are valid start server action
        if (
            valid["email"] == "valid" &&
            valid["password"] == "valid" &&
            valid["confirmPassword"] == "valid"
        ) {
            formAction(formData);
        }
    };

    return (
        <Form action={validateForm}>
            <FormHeader>Create an account</FormHeader>
            <FormError>{state.message}</FormError>
            <EmailInput
                isValid={valid.email == "valid"}
                isInvalid={valid.email == "invalid"}
            />
            <PasswordInput
                isValid={valid.password == "valid"}
                isInvalid={valid.password == "invalid"}
            />
            <PasswordInput
                name="confirmPassword"
                label="Confirm Password"
                invalidFeedback="Passwords do not match!"
                isValid={valid.confirmPassword == "valid"}
                isInvalid={valid.confirmPassword == "invalid"}
            />
            <div className="d-flex align-items-center text-center mb-3 flex-column">
                <SubmitButton style={{ width: "90%" }}>Sign up</SubmitButton>
            </div>
            <div className="d-flex align-items-center justify-content-center">
                <p className="mb-0 me-2">Already have an account?</p>
                <Link href="/login">
                    <SecondaryButton>Login</SecondaryButton>
                </Link>
            </div>
        </Form>
    );
}
